﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Web.Script.Serialization;
using System.Xml.Serialization;
using System.Xml;
using System.IO;
using System.Net;
using System.Net.Http;
using Newtonsoft.Json;

namespace Property_V4
{

    public partial class Form1 : Form
    {
        const string BASE_URL = "https://property.melissadata.net/v4/WEB/";
        private static readonly HttpClient client = new HttpClient();

        public Form1()
        {
            InitializeComponent();

            lookupPropertyButton.CheckedChanged += new EventHandler(endpointButtons_Changed);
            lookupDeedsButton.CheckedChanged += new EventHandler(endpointButtons_Changed);
            lookupHomesButton.CheckedChanged += new EventHandler(endpointButtons_Changed);

            endpointButtons_Changed(null, null);

            webBrowser1.AllowNavigation = true;
        }

        private void endpointButtons_Changed(object sender, EventArgs e)
        {
            if (lookupPropertyButton.Checked)
            {
                freeformBox.Enabled = true;
                fipsBox.Enabled = true;
                apnBox.Enabled = true;
                accountBox.Enabled = true;
                makBox.Enabled = true;
                addressKeyBox.Enabled = true;
                transactionIdBox.Enabled = false;
            }
            else if (lookupDeedsButton.Checked)
            {
                freeformBox.Enabled = true;
                fipsBox.Enabled = true;
                apnBox.Enabled = true;
                accountBox.Enabled = false;
                makBox.Enabled = true;
                addressKeyBox.Enabled = true;
                transactionIdBox.Enabled = true;
            }
            else if (lookupHomesButton.Checked)
            {
                freeformBox.Enabled = true;
                fipsBox.Enabled = false;
                apnBox.Enabled = false;
                accountBox.Enabled = false;
                makBox.Enabled = true;
                addressKeyBox.Enabled = false;
                transactionIdBox.Enabled = false;
            }
        }

        private void restButton_Click(object sender, EventArgs e)
        {
            string url = BASE_URL;
            if (lookupPropertyButton.Checked)
            {
                url += "LookupProperty";
                url += "?id=" + WebUtility.UrlEncode(customerIdBox.Text);
                url += "&t=" + WebUtility.UrlEncode("Property V4 - LookupProperty Test");
                url += "&cols=GrpAll";
                url += "&addresskey=" + WebUtility.UrlEncode(addressKeyBox.Text);
                url += "&fips=" + WebUtility.UrlEncode(fipsBox.Text);
                url += "&apn=" + WebUtility.UrlEncode(apnBox.Text);
                url += "&account=" + WebUtility.UrlEncode(accountBox.Text);
                url += "&mak=" + WebUtility.UrlEncode(makBox.Text);
                url += "&ff=" + WebUtility.UrlEncode(freeformBox.Text);
            }
            else if (lookupDeedsButton.Checked)
            {
                url += "LookupDeeds";
                url += "?id=" + WebUtility.UrlEncode(customerIdBox.Text);
                url += "&t=" + WebUtility.UrlEncode("Property V4 - LookupDeeds Test");
                url += "&cols=GrpAll";
                url += "&fips=" + WebUtility.UrlEncode(fipsBox.Text);
                url += "&apn=" + WebUtility.UrlEncode(apnBox.Text);
                url += "&txid=" + WebUtility.UrlEncode(transactionIdBox.Text);
                url += "&mak=" + WebUtility.UrlEncode(makBox.Text);
                url += "&ff=" + WebUtility.UrlEncode(freeformBox.Text);
            }
            else if (lookupHomesButton.Checked)
            {
                url += "LookupHomesByOwner";
                url += "?id=" + WebUtility.UrlEncode(customerIdBox.Text);
                url += "&t=" + WebUtility.UrlEncode("Property V4 - LookupHomesByOwner Test");
                url += "&cols=GrpAll";
                url += "&mak=" + WebUtility.UrlEncode(makBox.Text);
                url += "&ff=" + WebUtility.UrlEncode(freeformBox.Text);
            }

            if (jsonButton.Checked)
            {
                url += "&format=json";
            }
            else if (xmlButton.Checked)
            {
                url += "&format=xml";
            }

            requestBox.Text = url;

            doGetLookup(url);
        }

        private async void doGetLookup(string url)
        {
            var response = await client.GetAsync(url);
            if (response.IsSuccessStatusCode)
            {
                var content = await response.Content.ReadAsStringAsync();

                var path = Path.GetTempPath();

                if (url.Contains("format=xml"))
                {
                    var fullFileName = Path.Combine(path, "request.xml");
                    File.WriteAllText(fullFileName, content);
                    webBrowser1.Navigate(fullFileName);
                }
                else
                {
                    var t = JsonConvert.DeserializeObject<object>(content);
                    string jsonFormatted = JsonConvert.SerializeObject(t, Newtonsoft.Json.Formatting.Indented);
                    var fullFileName = Path.Combine(path, "request.json");
                    File.WriteAllText(fullFileName, jsonFormatted);
                    webBrowser1.Navigate(fullFileName);
                }

                tabControl1.SelectTab("responseTab");
            }
        }

        private async void doPostLookup(string endpoint, string encoding) {
            var request = new StringContent(requestBox.Text, Encoding.UTF8, encoding);
            var response = await client.PostAsync(BASE_URL + endpoint, request);
            var content = await response.Content.ReadAsStringAsync();

            var path = Path.GetTempPath();

            if (encoding.Equals("application/xml"))
            {
                var fullFileName = Path.Combine(path, "request.xml");
                File.WriteAllText(fullFileName, content);
                webBrowser1.Navigate(fullFileName);
            }
            else
            {
                var t = JsonConvert.DeserializeObject<object>(content);
                string jsonFormatted = JsonConvert.SerializeObject(t, Newtonsoft.Json.Formatting.Indented);
                var fullFileName = Path.Combine(path, "request.json");
                File.WriteAllText(fullFileName, jsonFormatted);
                webBrowser1.Navigate(fullFileName);
            }

            tabControl1.SelectTab("responseTab");
        }

        private string writeToXml(object lookup)
        {
            var serializer = new XmlSerializer(lookup.GetType());
            using (var sw = new StringWriter())
            {
                var settings = new XmlWriterSettings();
                settings.OmitXmlDeclaration = true;
                settings.Indent = true;
                settings.NewLineOnAttributes = true;

                using (var writer = XmlWriter.Create(sw, settings))
                {
                    serializer.Serialize(writer, lookup);
                    return sw.ToString();
                }
            }
        }

        private void postButton_Click(object sender, EventArgs e)
        {
            if (lookupPropertyButton.Checked)
            {
                var lookup = new LookupPropertyRequest();
                lookup.TransmissionReference = "Property V4 - LookupProperty Test";
                lookup.CustomerId = customerIdBox.Text;
                lookup.TotalRecords = "1";
                lookup.Columns = "grpAll";
                lookup.Records = new List<LookupPropertyRequest.RequestRecord>();

                var record = new LookupPropertyRequest.RequestRecord();
                record.RecordID = "1";
                record.AddressKey = addressKeyBox.Text;
                record.FIPS = fipsBox.Text;
                record.APN = apnBox.Text;
                record.Account = accountBox.Text;
                record.MAK = makBox.Text;
                record.FreeForm = freeformBox.Text;

                lookup.Records.Add(record);

                if (jsonButton.Checked)
                {
                    var serializer = new JavaScriptSerializer();
                    requestBox.Text = serializer.Serialize(lookup);
                }
                else if (xmlButton.Checked)
                {
                    requestBox.Text = writeToXml(lookup);
                }

                doPostLookup("LookupProperty", jsonButton.Checked ? "application/json" : "application/xml");
            }
            else if (lookupDeedsButton.Checked)
            {
                var lookup = new LookupDeedsRequest();
                lookup.TransmissionReference = "Property V4 - LookupDeeds Test";
                lookup.CustomerId = customerIdBox.Text;
                lookup.FreeForm = freeformBox.Text;
                lookup.MAK = makBox.Text;
                lookup.FIPS = fipsBox.Text;
                lookup.APN = apnBox.Text;
                lookup.TransactionID = transactionIdBox.Text;
                lookup.Columns = "grpAll";

                if (jsonButton.Checked)
                {
                    var serializer = new JavaScriptSerializer();
                    requestBox.Text = serializer.Serialize(lookup);
                }
                else if (xmlButton.Checked)
                {
                    requestBox.Text = writeToXml(lookup);
                }

                doPostLookup("LookupDeeds", jsonButton.Checked ? "application/json" : "application/xml");
            }
            else if (lookupHomesButton.Checked)
            {
                var lookup = new LookupHomesByOwnerRequest();
                lookup.TransmissionReference = "Property V4 - LookupHomesByOwner Test";
                lookup.CustomerId = customerIdBox.Text;
                lookup.OwnerMAK = makBox.Text;
                lookup.FreeForm = freeformBox.Text;

                if (jsonButton.Checked)
                {
                    var serializer = new JavaScriptSerializer();
                    requestBox.Text = serializer.Serialize(lookup);
                }
                else if (xmlButton.Checked)
                {
                    requestBox.Text = writeToXml(lookup);
                }

                doPostLookup("LookupHomesByOwner", jsonButton.Checked ? "application/json" : "application/xml");
            }
        }

        private void clearButton_Click(object sender, EventArgs e)
        {
            freeformBox.Text = "";
            fipsBox.Text = "";
            apnBox.Text = "";
            accountBox.Text = "";
            makBox.Text = "";
            addressKeyBox.Text = "";
            transactionIdBox.Text = "";
        }

        public class LookupPropertyRequest
        {
            public string TransmissionReference;
            public string CustomerId;
            public string Columns;
            public string TotalRecords;
            public List<RequestRecord> Records;

            public class RequestRecord
            {
                public string RecordID;
                public string AddressKey;
                public string FIPS;
                public string APN;
                public string Account;
                public string MAK;
                public string FreeForm;
            }
        }

        public class LookupDeedsRequest
        {
            public string TransmissionReference;
            public string CustomerId;
            public string Columns;
            public string MAK;
            public string FIPS;
            public string APN;
            public string TransactionID;
            public string FreeForm;
        }

        public class LookupHomesByOwnerRequest
        {
            public string TransmissionReference;
            public string CustomerId;
            public string OwnerMAK;
            public string FreeForm;
        }
    }
}
